require 'spec_helper'

describe Palantir do
  context "created with some data" do
    some_data = [
      {foo: 'bar'},
      {bar: 'foo'},
      {name: 'foobar', value: 42},
      {name: 'foobar', value: 21},
    ]
    palantir = Palantir.new(some_data)

    it "return the 'count' of values" do
      expect(palantir.size).to eq 4
    end

    it "has a 'where' clause that filter on the content" do
      expect(palantir.where(:foo, 'bar').size).to eq 1
    end

    context "the 'where' clause" do
      it "is chainable" do
        expect(palantir.where(:name, 'foobar').where(:value, 42).size).to eq 1
      end

      it "found the right element" do
        expect(palantir.where(:name, 'foobar').where(:value, 42).data).to match_array [{name: 'foobar', value: 42}]
      end

      it "found nothing if there is nothing to find" do
        expect(palantir.where(:name, 'nothing').where(:value, 42).data).to match_array []
      end

      it "found nothing if you have no idea to what looking for" do
        expect(palantir.where(:key_not_found, 'nothing').where(:something_strange, 0).data).to match_array []
      end
    end

    context "the 'sum' function" do
      context "understand a basic sum operation" do

        it "returns the correct value with a 'where' clause" do
          expect(palantir.where(:name, 'foobar').sum(:value)).to eq 63
        end

        it "returns the default value with a missing value to sum" do
          expect(palantir.where(:foo, 'bar').sum(:key_not_found)).to eq 0
        end

        it "returns the default value if there is nothing to count" do
          expect(palantir.where(:key_not_found, 'nothing').where(:something_strange, 0).sum(:value)).to eq 0
        end

        it "returns the default value if there is not a value" do
          expect(palantir.where(:key_not_found, 'meh').sum(:key_not_found)).to eq 0
        end

        it "returns the default value if there is not the searched key" do
          expect(palantir.where(:name, 'meh').sum(:value)).to eq 0
        end
      end
    end
  end
end
